#include <stdio.h>
#include <time.h>
#include <math.h>
#include "PsxIO.h"
#include "comm.h"

/*
 * Ein/Ausgabekanaele
 */

#define PSX_IN	0
#define PSX_OUT 1
#define PSX_ERR 2

/*
 * Berechnen der Funktion.
 */

double f( double x )
{
    return ( sin(x) );
			 
} /* end of f() */

/*
 * Integral eines Intervalls. Total simpel. Der Wert f((x+y)/2) wird mit
 * der Breite des Intervalles multipliziert und alles wird aufsummiert.
 */

double eval(int intervals, double start, double end)
{
    int     current;
    double  width, sum;
						 
    sum   = 0.0;
    width = (end-start) / (double)intervals;
								 
    for (current = 0; current < intervals; current++)
	sum += f (start + ((0.5 + (double) current) * width));

    return ( sum * width );
} /* end of eval() */


int main(void)
{
    int prozessnr=0, ok=1, res, myintval=0;
    double mystart, myend, result;
    Comm MyComm;
	
    while (ok) {	
	res = PsxRead(PSX_IN,(char *) &MyComm,sizeof(Comm));

	if (prozessnr == 0) { 			/* expecting COUNT */
	    if(MyComm.command == COUNT) {
		prozessnr = ++MyComm.u.count;
		res = PsxWrite(PSX_OUT,(char *) &MyComm,sizeof(Comm));
	    }	
	    else
	      fprintf (stderr, "Count erwartet Slave ??:%d\n", __LINE__);
	}
	else {
	    if (prozessnr == MyComm.process) { 	/* Befehl an mich! */
		switch (MyComm.command) {
		case AUFGABE: 			/* Aufgabe erhalten. Merken */
		    mystart  = MyComm.u.aufg.anf;
		    myend    = MyComm.u.aufg.end;
		    myintval = MyComm.u.aufg.schritt;

		    res = PsxWrite(PSX_OUT,(char *) &MyComm,sizeof(Comm));
		    break;
		case ERGEBNIS:			/* Ergebnis wird abgefragt */
		    MyComm.u.ergebnis = result;
		    res = PsxWrite(PSX_OUT,(char *) &MyComm,sizeof(Comm));
		    break;
		default:
		    fprintf (stderr, "Ungueltiges Kommando Slave %d:%d\n",
			     prozessnr, __LINE__);
		    break;
		}
	    }
	    else if (MyComm.process == 0) {	/* Broadcast */
		/*
		 * Broadcast erst weiterleiten, dann bearbeiten
		 */
		res = PsxWrite (PSX_OUT, (char *) &MyComm, sizeof (Comm));

		switch (MyComm.command) {
		case GO:			/* Berechnung starten */
		    result = eval (myintval, mystart, myend);
		    break;
		case QUIT:			/* Alles beenden */
		    ok = 0;
		    break;
		default:
		    fprintf (stderr, "Ungueltiges Kommando Slave %d:%d\n",
			     prozessnr, __LINE__);
		    break;
		}
	    }
	    else
	      res = PsxWrite(PSX_OUT,(char *) &MyComm,sizeof(Comm));
	}
    } /* While ok */
}








